/*
 * This Billy-Bass-motors sketch operates the motors in a 2-motor Bigmouth 
 * Billy Bass, controlled by a Samsung TV remote. The brand of the remote can 
 * be changed easily. The sketch includes movements that sync with 3 comments 
 * spoken by Billy in the Billy-Bass-comments sketch.    June 2024.
 *
 * It is based on the SimpleReceiver example in the IRremote library. The example 
 * is part of Arduino-IRremote https://github.com/Arduino-IRremote/Arduino-IRremote.
 *
 * A few changes are made to the example for this project: Changed NEC to SAMSUNG,
 * changed Serial.begin to 9600, changed to not display data for UNKNOWN protocol.
 *
 ************************************************************************************
 * MIT License
 *
 * Copyright (c) 2020-2023 Armin Joachimsmeyer. 
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is furnished
 * to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 ************************************************************************************
 */

/*
 * Specify which protocol(s) should be used for decoding.
 * If no protocol is defined, all protocols (except Bang&Olufsen) are active.
 * This must be done before the #include <IRremote.hpp>
 */
//#define DECODE_DENON        // Includes Sharp
//#define DECODE_JVC
//#define DECODE_KASEIKYO
//#define DECODE_PANASONIC    // alias for DECODE_KASEIKYO
//#define DECODE_LG
//#define DECODE_NEC          // Includes Apple and Onkyo. To enable all protocols , just comment/disable this line.
#define DECODE_SAMSUNG
//#define DECODE_SONY
//#define DECODE_RC5
//#define DECODE_RC6

//#define DECODE_BOSEWAVE
//#define DECODE_LEGO_PF
//#define DECODE_MAGIQUEST
//#define DECODE_WHYNTER
//#define DECODE_FAST

//#define DECODE_DISTANCE_WIDTH // Universal decoder for pulse distance width protocols
//#define DECODE_HASH         // special decoder for all protocols

//#define DECODE_BEO          // This protocol must always be enabled manually, i.e. it is NOT enabled if no protocol is defined. It prevents decoding of SONY!

//#define DEBUG               // Activate this for lots of lovely debug output from the decoders.

//#define RAW_BUFFER_LENGTH  180  // Default is 112 if DECODE_MAGIQUEST is enabled, otherwise 100.

#include <Arduino.h>
#include <AFMotor.h>            // Attach the motor library.

AF_DCMotor headtail(3);         // Specify which motor ports are used for head/tail motor
AF_DCMotor mouth(4);            // and mouth motor.

/*
 * This include defines the actual pin number for pins like IR_RECEIVE_PIN, IR_SEND_PIN for many different boards and architectures
 */
#include "PinDefinitionsAndMore.h"
#include <IRremote.hpp> // include the IRlibrary

void setup() {
    Serial.begin(9600);
    // Just to know which program is running on my Arduino
    Serial.println(F("START " __FILE__ " from " __DATE__ "\r\nUsing library version " VERSION_IRREMOTE));

    // Start the receiver and if not 3. parameter specified, take LED_BUILTIN pin from the internal boards definition as default feedback LED
    IrReceiver.begin(IR_RECEIVE_PIN, ENABLE_LED_FEEDBACK);

    Serial.print(F("Ready to receive IR signals of protocols: "));
    printActiveIRProtocols(&Serial);
    Serial.println(F("at pin " STR(IR_RECEIVE_PIN)));

    headtail.setSpeed(255);     // Set speed of motors. Full speed is 255.
    mouth.setSpeed(255);
}

void lips(int opentime, int closedtime, int times){
  for (int i=0; i<times; i++){    // Loop for number of times.
    mouth.run(FORWARD);           // Turn motor on, to open the mouth.
    delay(opentime);              // Wait.
    mouth.run(RELEASE);           // Turn motor off.
    delay(200);                   // Wait while motor moves.
    mouth.run(BACKWARD);          // Turn motor on, to close the mouth.
    delay(200);                   // Wait while motor moves.
    mouth.run(RELEASE);           // Turn motor off.
    delay(closedtime);            // Wait.
  }
}

void flaptail(int times){
  for (int i=0; i<times; i++){    // Loop for number of times.
    Serial.println("--flap tail--");  // Display in Serial Monitor.
    headtail.run(BACKWARD);       // Turn motor on, to bend the tail.
    delay(300);                   // Wait while motor moves.
    headtail.run(RELEASE);        // Turn motor off.
    delay(300);                   // Wait while motor moves.           
  }
}

void loop() {
    /*
     * Check if received data is available and if yes, try to decode it.
     * Decoded result is in the IrReceiver.decodedIRData structure.
     *
     * E.g. command is in IrReceiver.decodedIRData.command
     * address is in command is in IrReceiver.decodedIRData.address
     * and up to 32 bit raw data in IrReceiver.decodedIRData.decodedRawData
     */
    if (IrReceiver.decode()) {

        /*
         * Print a summary of received data
         */
        if (IrReceiver.decodedIRData.protocol == UNKNOWN) {
          //  Serial.println(F("Received noise or an unknown (or not yet enabled) protocol"));
            // We have an unknown protocol here, print extended info
          //  IrReceiver.printIRResultRawFormatted(&Serial, true);
              IrReceiver.resume(); // Do it here, to preserve raw data for printing with printIRResultRawFormatted()
        } else {
            IrReceiver.resume(); // Early enable receiving of the next IR frame
            IrReceiver.printIRResultShort(&Serial);
          //  IrReceiver.printIRSendUsage(&Serial);
        }
      //  Serial.println();

        IrReceiver.stop();     //   ******* Needed to avoid TIMER conflict

        /*
         * Finally, check the received data and perform actions according to the received command
         */
        if (IrReceiver.decodedIRData.command == 0x7) {
          Serial.println(". Open the mouth.");
          mouth.run(RELEASE);       // Turn motor off, in case it was on.
          delay(200);               // Wait while motor moves.
          mouth.run(FORWARD);       // Turn motor on, to open the mouth.
          delay(200);               // Wait while motor moves.
        }
        if (IrReceiver.decodedIRData.command == 0xB) {
          Serial.println(". Close the mouth.");
          mouth.run(RELEASE);       // Turn motor off.
          delay(200);               // Wait while motor moves.
          mouth.run(BACKWARD);      // Turn motor on, to close the mouth.
          delay(200);               // Wait while motor moves.
          mouth.run(RELEASE);       // Turn motor off. 
        }
        if (IrReceiver.decodedIRData.command == 0xF) {
          Serial.println(". Turn the head.");
          headtail.run(RELEASE);    // Turn motor off, in case it was on.
          delay(400);               // Wait while motor moves.
          headtail.run(FORWARD);    // Turn motor on, to turn the head.
          delay(400);               // Wait while motor moves.
        }
        if (IrReceiver.decodedIRData.command == 0x6B) {
          Serial.println(". Return the head.");
          headtail.run(RELEASE);    // Turn motor off.
          delay(400);               // Wait while motor moves.
        }
        if (IrReceiver.decodedIRData.command == 0x12) {
          Serial.println(". Bend the tail.");
          headtail.run(RELEASE);    // Turn motor off, in case it was on.
          delay(300);               // Wait while motor moves.
          headtail.run(BACKWARD);   // Turn motor on, to bend the tail.
          delay(300);               // Wait while motor moves.
        }
        if (IrReceiver.decodedIRData.command == 0x10) {
          Serial.println(". Return the tail.");
          headtail.run(RELEASE);    // Turn motor off.
          delay(300);               // Wait while motor moves.
        }
        if (IrReceiver.decodedIRData.command == 0x8C) {
          Serial.println(". Say Hello.");
          headtail.run(FORWARD);    // Turn motor on, to turn the head.
          delay(400);               // Wait while motor moves.
          lips(500, 500, 1);        // Open mouth for 1/2 sec and close for 1/2 sec.
          headtail.run(RELEASE);    // Turn motor off. A spring returns the head.
          delay(400);               // Wait while motor moves.
          flaptail(2);              // Flap tail twice.
        }
       if (IrReceiver.decodedIRData.command == 0x1A) {
          Serial.println(". Say Good to see you.");
          headtail.run(FORWARD);    // Turn motor on, to turn the head.
          delay(400);               // Wait while motor moves.
          lips(1000, 500, 1);       // Open mouth for 1 sec and close for 1/2 sec.
          headtail.run(RELEASE);    // Turn motor off. A spring returns the head.
          delay(400);               // Wait while motor moves.
        }
       if (IrReceiver.decodedIRData.command == 0x3) {
          Serial.println(". Say Bye.");
          headtail.run(FORWARD);    // Turn motor on, to turn the head.
          delay(400);               // Wait while motor moves.
          lips(500, 500, 1);        // Open mouth for 1/2 sec and close for 1/2 sec.
          headtail.run(RELEASE);    // Turn motor off. A spring returns the head.
          delay(400);               // Wait while motor moves.
          flaptail(2);              // Flap tail twice.
        }

        IrReceiver.start(8000);     //   ******* Needed to avoid TIMER conflict 
    }
}
